/**********************************************************************
 *
 * documentaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: documentaction.cpp 450 1999-08-11 23:28:03Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.21  1999/08/11 23:28:03  sjboddie
   -added support for html classifier
   - removed _random_ macro - now use _blank for detach page

   Revision 1.20  1999/08/10 23:16:39  sjboddie
   a couple of small changes to get format options DocumentArrowsBottom
   and DocumentArrowsTop working correctly

   Revision 1.19  1999/08/10 22:40:01  sjboddie
   added some more format options including the ability to format the
   document text

   Revision 1.18  1999/08/09 04:25:18  sjboddie
   moved OID translation stuff from documentaction::define_external_macros
   to receptionist

   Revision 1.17  1999/08/09 02:13:53  sjboddie
   fixed small bug in ShowSectionTitles

   Revision 1.16  1999/08/03 03:30:47  sjboddie
   a few small changes related to new format options

   Revision 1.15  1999/07/30 02:18:49  sjboddie
   -added collectinfo argument to some functions
   -caught up with changes to browsetools (allowing nested classifications)
   -added showtoppage format option

   Revision 1.14  1999/07/21 05:01:09  sjboddie
   fixed up wrapping of classificationlinks

   Revision 1.13  1999/07/20 02:59:54  sjboddie
   List and AZList classifications now use format strings

   Revision 1.12  1999/07/16 00:19:01  sjboddie
   some changes to the way quoted queries are handled

   Revision 1.11  1999/07/07 05:47:41  sjboddie
   changed around the way browsetools works

   Revision 1.10  1999/06/29 01:46:44  sjboddie
   now sets a _navigationbar_ macro even if there aren't any
   classifications (it will just have the 'search' button

   Revision 1.9  1999/06/27 22:05:58  sjboddie
   now set a _thisOID_ macro for use in displaying images

   Revision 1.8  1999/06/24 05:12:20  sjboddie
   lots of small changes

   Revision 1.7  1999/06/17 03:06:58  sjboddie
   got detach button working properly - the close book icon is now disabled
   when page is detached as the javascript close() function I was using is
   too unreliable over different browsers
   note that in my last comment I meant the "cl" arg (not the "c" arg).

   Revision 1.6  1999/06/16 23:53:15  sjboddie
   tidied a few things up. documentaction::define_external_macros now
   resets the "c" arg if it's set to something stupid by the .xx suffixes

   Revision 1.5  1999/06/16 03:10:49  sjboddie
   define_internal_macros() now sets _pagetitle_ macro to document's title
   (including parents of current document)

   Revision 1.4  1999/06/15 01:55:29  sjboddie
   - got text highlighting working
   - got multiple collections working (now outputs error message if an
     attempt is made to get a document when the "c" arg isn't set.

   Revision 1.3  1999/06/08 04:29:37  sjboddie
   added argsinfo to the call to check_cgiargs to make it easy to set
   args to their default if they're found to be screwed up

   Revision 1.2  1999/05/10 03:40:35  sjboddie
   lots of changes - slowly getting document action sorted out

   Revision 1.1  1999/04/30 01:59:40  sjboddie
   lots of stuff - getting documentaction working (documentaction replaces
   old browseaction)

   Revision 1.2  1999/03/29 02:14:27  sjboddie

   More changes to browseaction

   Revision 1.1  1999/03/25 03:09:40  sjboddie

   subjectbrowseaction became browseaction


 */


#include <string.h>
#include "documentaction.h"
#include "browsetools.h"
#include "OIDtools.h"
#include "querytools.h"
#include "unitool.h"

documentaction::documentaction () {
  // this action uses cgi variables "a", "d", "cl",
  // "x", "gc", "gt", "gp", and "hl"
  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "a";
  arg_ainfo.longname = "action";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "p";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "d";
  arg_ainfo.longname = "document OID";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "cl";
  arg_ainfo.longname = "classification OID";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gc" controls the expand/contract
  // contents function
  arg_ainfo.shortname = "gc";
  arg_ainfo.longname = "expand contents";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gt" controls the expand/contract
  // text function 0 = not expanded, 1 = expand unless 
  // there are more than 10 sections containing text,
  // 2 = expand all
  arg_ainfo.shortname = "gt";
  arg_ainfo.longname = "expand text";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gp" is the "go to page" control
  // used by the Book type of toc
  arg_ainfo.shortname = "gp";
  arg_ainfo.longname = "go to page";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::mustnot;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "hl" is the "highlighting on/
  // highlighting off control
  arg_ainfo.shortname = "hl";
  arg_ainfo.longname = "highlighting on/off";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "1";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // "x" is 0 normally or 1 if page
  // has been "detached"
  arg_ainfo.shortname = "x";
  arg_ainfo.longname = "detached page";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);
}

documentaction::~documentaction () {
}

bool documentaction::init (ostream &logout) {
  return action::init (logout);
}

bool documentaction::check_cgiargs (cgiargsinfoclass &argsinfo, cgiargsclass &args, 
				    ostream &logout) {

  // note that "gp" argument gets checked in define_external_macros()

  // check gc argument
  int arg_gc = args.getintarg("gc");
  if (arg_gc != 0 && arg_gc != 1) {
    logout << "Warning: \"gc\" argument out of range (" << arg_gc << ")\n";
    cgiarginfo *gcinfo = argsinfo.getarginfo ("gc");
    if (gcinfo != NULL) args["gc"] = gcinfo->argdefault;
  }

  // check gt argument (may be either 0, 1 or 2)
  int arg_gt = args.getintarg("gt");
  if (arg_gt != 0 && arg_gt != 1 && arg_gt != 2) {
    logout << "Warning: \"gt\" argument out of range (" << arg_gt << ")\n";
    cgiarginfo *gtinfo = argsinfo.getarginfo ("gt");
    if (gtinfo != NULL) args["gt"] = gtinfo->argdefault;
  }

  // check hl argument
  int arg_hl = args.getintarg("hl");
  if (arg_hl != 0 && arg_hl != 1) {
    logout << "Warning: \"hl\" argument out of range (" << arg_hl << ")\n";
    cgiarginfo *hlinfo = argsinfo.getarginfo ("hl");
    if (hlinfo != NULL) args["hl"] = hlinfo->argdefault;
  }

  // check x argument
  int arg_x = args.getintarg("x");
  if (arg_x != 0 && arg_x != 1) {
    logout << "Warning: \"x\" argument out of range (" << arg_x << ")\n";
    cgiarginfo *xinfo = argsinfo.getarginfo ("x");
    if (xinfo != NULL) args["x"] = xinfo->argdefault;
  }

  return true;
}

void documentaction::get_cgihead_info (cgiargsclass &/*args*/, response_t &response,
				     text_t &response_data, ostream &/*logout*/) {
  response = content;
  response_data = "text/html";
}


// define all the macros which might be used by other actions
// to produce pages. 
void documentaction::define_external_macros (const ColInfoResponse_t &/*collectinfo*/, displayclass &disp, 
					     cgiargsclass &args, recptproto *collectproto, 
					     ostream &logout) {
  
  // define_external_macros sets the following macros:

  // _navigationbar_     this is the navigation bar containing the search button
  //                     and any classification buttons - it goes at the top of 
  //                     most pages. for now we're assuming that there'll always 
  //                     be a search button - we should probably check that there
  //                     is a query action before making this assumption

  // _httpbrowseXXX_     the http macros for each classification (i.e. if there
  //                     are Title and Creator classifications _httpbrowseTitle_
  //                     and _httpbrowseCreator_ will be set

  // _javaimagesnavbar_  this is the javascript code to shove in to make the 
  //                     flashy images used by _navigationbar_ work
  

  // can't do anything if collectproto is null (i.e. no collection was specified)
  if (collectproto == NULL) return;

  outconvertclass text_t2ascii;
  comerror_t err;
  InfoFiltersResponse_t filterinfo;
  FilterResponse_t response;
  text_tarray metadata;
  text_t navigationbar, javaimagesnavbar, width, topparent;
  int twidth, swidth, iwidth = 0;

  text_t &arg_d = args["d"];
  text_t &arg_cl = args["cl"];
  text_t &collection = args["c"];

  // don't want navigation bar if page is 'detached'
  if (!args.getintarg("x")) {

    get_top (arg_cl, topparent);
    
    collectproto->get_filterinfo (collection, filterinfo, err, logout);
    if (err == noError) {
      // check that there's a browse filter
      if (filterinfo.filterNames.find ("BrowseFilter") != filterinfo.filterNames.end()) {
	
	metadata.push_back ("Title");
	metadata.push_back ("classifytype");
	bool getParents = false;
	get_children ("", collection, metadata, getParents, collectproto, response, logout);
	  
	int numc = response.docInfo.size();
	ResultDocInfo_tarray::const_iterator dochere = response.docInfo.begin();
	ResultDocInfo_tarray::const_iterator docend = response.docInfo.end();
	
	navigationbar = "<!-- Navigation Bar -->\n";
	
	// calculate width of spacers and set macro
	if (args.getintarg("v") == 0) {
	  disp.expandstring ("Global", "_pagewidth_", width);
	  twidth = width.getint();
	  
	  disp.expandstring ("query", "_searchwidth_", width);
	  iwidth += width.getint();
	  
	  while (dochere != docend) {
	    disp.expandstring ("document", "_" + (*dochere).metadata[0].values[0] + "width_", width);
	    iwidth += width.getint();
	    dochere ++;
	  }
	  if ((twidth - iwidth) < numc) swidth = 2;
	  else {
	    swidth = twidth - iwidth;
	    if (numc > 0) swidth = swidth / numc;
	  }
	  disp.setmacro ("widthtspace", "Global", swidth);
	}
	
	navigationbar += "<nobr>\n";
	if (args["a"] == "q") {
	  navigationbar += "_icontabsearchgreen_";
	} else {
	  navigationbar += "_imagesearch_";
	  javaimagesnavbar = "_javasearch_";
	}
	if (numc == 0) navigationbar += "_imagespacer_";
	
	dochere = response.docInfo.begin();
	while (dochere != docend) {
	  const text_t &title = (*dochere).metadata[0].values[0];
	  const text_t &classifytype = (*dochere).metadata[1].values[0];
	  
	  // if we're inside a document all the classification buttons should be enabled
	  if (arg_d.empty() && ((*dochere).OID == topparent))
	    navigationbar += "_imagespacer__icontab" + title + "green_";
	  else {
	    navigationbar += "_imagespacer__image" + title + "_";
	    
	    // set the _httpbrowseXXX_ macro for this classification
	    text_t link;
	    if (classifytype == "HTML") 
	      link = "_httppagex_(html)&hp=" + (*dochere).OID + ".fc";
	    else 
	      link = "_httpdocument_&cl=" + (*dochere).OID;

	    if (classifytype == "AZList" || classifytype == "DateList") link += ".fc";
	    disp.setmacro ("httpbrowse" + title, "Global", link); 

	  }
	    javaimagesnavbar += "_java" + title + "_";
	    dochere ++;
	}
	navigationbar += "\n</nobr>\n";
	navigationbar += "<!-- End of Navigation Bar -->\n";
	disp.setmacro ("navigationbar", "Global", navigationbar);
	if (args.getintarg("v") == 0) 
	  disp.setmacro ("javaimagesnavbar", "Global", javaimagesnavbar);
      }
    } else {
      logout << text_t2ascii 
	     << "Error (documentaction::define_external_macros()) in call to get_filterinfo() " 
	     << get_comerror_string (err);
    }
  }
}


// define all the macros which are related to pages generated
// by this action. we also load up the formatinfo structure 
// here (it's used in do_action as well as here)
void documentaction::define_internal_macros (const ColInfoResponse_t &collectinfo, displayclass &disp, 
					     cgiargsclass &args, recptproto *collectproto, 
					     ostream &logout) {

  // define_internal_macros sets the following macros:

  // _pagetitle_            the title to be displayed at the top of the browser window
  
  // _imagethispage_        the title image to be displayed at top right of page

  // _classificationlinks_  the links between classifications to be displayed
  //                        at top of list or datelist classifications

  // _httpprevarrow_        these are set if next or previous arrows
  // _httpnextarrow_        are to be used - (i.e. if it's an AZList or a DateList 
  //                        classifytype at a top level or if it's a Book or a
  //                        Hierarchy classifytype at document level 

  // _navarrows_            this may be overridden to "" when format option 
  //                        DocumentArrowsBottom is false

  // _header_               the header macro is overridden if we're not at a top level 
  //                        classification to remove the title block

  // _javaimagescontent_    this is the javascript code to shove in to make the 
  //                        flashy buttons work

  // _thisOID_              the OID (directory) of the current document - this corresponds
  //                        to the archivedir metadata element

  // can't do anything if collectproto is null (i.e. no collection was specified)
  if (collectproto == NULL) return;

  // load up formatinfo
  formatinfo.clear();
  text_tmap::const_iterator format_here = collectinfo.format.begin();
  text_tmap::const_iterator format_end = collectinfo.format.end();

  while (format_here != format_end) {
    if (((*format_here).first == "DocumentImages") && 
	((*format_here).second == "true"))
      formatinfo.DocumentImages = true;
    else if (((*format_here).first == "DocumentTopPages") && 
	     ((*format_here).second == "false"))
      formatinfo.DocumentTopPages = false;
    else if ((*format_here).first == "DocumentHeading") 
      formatinfo.DocumentHeading = (*format_here).second;
    else if (((*format_here).first == "DocumentArrowsTop") && 
	     ((*format_here).second == "false"))
      formatinfo.DocumentArrowsTop = false;
    else if (((*format_here).first == "DocumentArrowsBottom") && 
	     ((*format_here).second == "false"))
      formatinfo.DocumentArrowsBottom = false;
    else if (((*format_here).first == "DocumentGoTo") && 
	     ((*format_here).second == "false"))
      formatinfo.DocumentGoTo = false;
    else if ((*format_here).first == "DocumentButtons")
      splitchar ((*format_here).second.begin(), (*format_here).second.end(),
		 '|', formatinfo.DocumentButtons);
    else if ((*format_here).first == "DocumentText") 
      formatinfo.DocumentText = (*format_here).second;

    format_here ++;
  }

  text_tarray metadata;
  FilterResponse_t response;
  text_t &arg_d = args["d"];
  text_t &arg_cl = args["cl"];
  text_t &collection = args["c"];

  if (!formatinfo.DocumentArrowsBottom)
    disp.setmacro("navarrows", "document", "");

  metadata.push_back ("Title");
  metadata.push_back ("classifytype");
  metadata.push_back ("archivedir");
  
  if (!arg_d.empty()) {
    // we're at document level
    text_t javaimagescontent;

    // get metadata for this document and it's parents
    if (get_info (arg_d, collection, metadata, true, collectproto, response, logout)) {
      disp.setmacro ("header", "document", "_textheader_");

      text_t pagetitle;
      if (args.getintarg("gt") == 2) 
	pagetitle = *(response.docInfo[0].metadata[0].values.begin());
      else {
	text_tarray::const_iterator this_title = response.docInfo[0].metadata[0].values.begin();
	text_tarray::const_iterator end_title = response.docInfo[0].metadata[0].values.end();
	while (this_title != end_title) {
	  if ((this_title + 1) == end_title) pagetitle += *this_title;
	  else pagetitle += *this_title + ": ";
	  this_title ++;
	}
      }
      disp.setmacro ("pagetitle", "document", pagetitle);
      disp.setmacro ("thisOID", "Global", response.docInfo[0].metadata[2].values[0]);

      text_t &classifytype = response.docInfo[0].metadata[1].values[0];
      if (classifytype.empty()) classifytype = "Book"; // defaults to Book

      // set arrow macros if required
      if (((classifytype == "Book") && (formatinfo.DocumentArrowsTop || formatinfo.DocumentArrowsBottom)) ||
	  ((classifytype == "Hierarchy") && formatinfo.DocumentArrowsBottom))
	set_arrow_macros (arg_d, classifytype, formatinfo.DocumentTopPages, 
			  disp, collectproto, collection, logout);
      
	  
      if (args["u"] != "1") {
	javaimagescontent = "_javaextras_";
	
	int arg_gt = args.getintarg("gt");
	int arg_gc = args.getintarg("gc");
	int arg_hl = args.getintarg("hl");

	text_tarray::const_iterator button_here = formatinfo.DocumentButtons.begin();
	text_tarray::const_iterator button_end = formatinfo.DocumentButtons.end();

	while (button_here != button_end) {
	  if (*button_here == "Detach") 
	    javaimagescontent += "_javadetach_";
	  else if (*button_here == "Expand Text") {
	    if (arg_gt == 1)
	      javaimagescontent += "_javacontracttext__javacontinue_";
	    else if (arg_gt == 2)
	      javaimagescontent += "_javacontracttext_";
	    else
	      javaimagescontent += "_javaexpandtext_";
	  } else if (*button_here == "Expand Contents") {
	      if (arg_gc == 1)
		javaimagescontent += "_javacontractcontents_";
	      else
		javaimagescontent += "_javaexpandcontents_";
	  } else if (*button_here == "Highlight") {
	    if (arg_hl == 1)
	      javaimagescontent += "_javanohighlighting_";
	    else 
	      javaimagescontent += "_javahighlighting_";
	  }
	  button_here ++;
	}
	disp.setmacro ("javaimagescontent", "document", javaimagescontent);
      }
    }
  } else {
    if (!arg_cl.empty()) {
	
      // get metadata for top level classification
      text_t classtop;
      get_top (arg_cl, classtop);
      if (get_info (classtop, collection, metadata, false, collectproto, response, logout)) {
	  
	text_t &title = response.docInfo[0].metadata[0].values[0];
	text_t &classifytype = response.docInfo[0].metadata[1].values[0];

	disp.setmacro ("pagetitle", "document", "_text" + title + "page_");

	disp.setmacro ("imagethispage", "document", "_icon" + title + "page_");

	// now get the metadata for each child of top level
	// so we can generate the _classificationlinks_ and arrow macros
	// (if they're required by the current classification type)
	
	if ((classifytype == "AZList") || (classifytype == "DateList")) {
	  
	  text_t classificationlinks, navarrows;
	  text_t link = "<a href=\"_httpdocument_&cl=";

	  metadata.erase(metadata.begin(), metadata.end());
	  metadata.push_back ("Title");
	  metadata.push_back ("doctype");
	  bool getParents = false;
	  if (get_children (classtop, collection, metadata, getParents, collectproto, response, logout)) {
	    
	    // don't want links unless there are 2 or more sections
	    if (response.docInfo.size() >= 2) {
	      
	      classificationlinks += "<!-- Classification Links -->\n";
	      classificationlinks += "<table width=_pagewidth_ cellpadding=0 cellspacing=0 border=0>\n";
	      classificationlinks += "<tr><td valign=top><center>\n";
	      
	      ResultDocInfo_tarray::const_iterator sechere = response.docInfo.begin();
	      ResultDocInfo_tarray::const_iterator secend = response.docInfo.end();
	      
	      int lenlinks = 0;
	      while (sechere != secend) {
		if ((*sechere).metadata[1].values[0] == "classify") {
		  const text_t &sectionheading = (*sechere).metadata[0].values[0];

		  if (((*sechere).OID == arg_cl) || is_child_of((*sechere).OID, arg_cl)) {
		    
		    // set the _httpprevarrow_, _httpnextarrow_ macros while we're here
		    if (formatinfo.DocumentArrowsBottom) {
		      if (sechere != response.docInfo.begin())
			disp.setmacro ("httpprevarrow", "document", "_httpdocument_&cl=" + (*(sechere-1)).OID);
		      if ((sechere + 1) != secend)
			disp.setmacro ("httpnextarrow", "document", "_httpdocument_&cl=" + (*(sechere+1)).OID);
		    }

		    lenlinks += sectionheading.size();
		    classificationlinks += "<b>" + sectionheading + "</b>";
		    if (lenlinks > 50) {
		      classificationlinks += "<br>\n";
		      lenlinks = 0;
		    } else {
		      if (sechere+1 != secend) classificationlinks += "&nbsp;&nbsp;&nbsp;\n";
		      lenlinks += 3;
		    }
		  } else {
		    lenlinks += sectionheading.size();
		    classificationlinks += link + (*sechere).OID + "\">";
		    classificationlinks += sectionheading + "</a>";
		    if (lenlinks > 50) {
		      classificationlinks += "<br>\n";
		      lenlinks = 0;
		    } else {
		      if (sechere+1 != secend) classificationlinks += "&nbsp;&nbsp;&nbsp;\n";
		      lenlinks += 3;
		    }
		  }
		}
		sechere ++;
	      }
	      
	      classificationlinks += "\n</center></td></tr></table>\n";
	      classificationlinks += "<!-- End of Classification Links -->\n";
	      
	      disp.setmacro ("classificationlinks", "document", classificationlinks);
	    }
	  }
	}
      }
    }
  }
}

bool documentaction::do_action (cgiargsclass &args, const ColInfoResponse_t &collectinfo,
				recptproto *collectproto, displayclass &disp, 
				outconvertclass &outconvert, ostream &textout, 
				ostream &logout) {

  if (collectproto == NULL) {
    logout << "documentaction::do_action called with NULL collectproto\n";
    textout << outconvert << disp << "_document:header_\n" 
	    << "Error: Attempt to get document without setting collection\n"
	    << "_document:footer_\n";
  } else {  

    text_tarray metadata;
    FilterResponse_t response;
    text_t toptitle, toptype, thistype, formatstring;

    text_t &collection = args["c"];
    text_t &arg_d = args["d"];
    text_t &arg_cl = args["cl"];
    
    text_t OID = arg_d;
    if (arg_d.empty()) OID = arg_cl;
    
    textout << outconvert << disp << "_document:header_\n"
	    << "_document:content_\n";
    
    if (arg_d.empty() && arg_cl.empty()) {
      textout << outconvert << disp << "Document contains no data_document:footer_\n";
      return true;
    }
    
    // get the classifytitle and classifytype for both the top level 
    // and the current level (using getParent=true)
    metadata.push_back ("Title");
    metadata.push_back ("classifytype");
    if (get_info (OID, collection, metadata, true, collectproto, response, logout)) {
      toptitle = response.docInfo[0].metadata[0].values[0];
      toptype = response.docInfo[0].metadata[1].values[0];
      thistype = response.docInfo[0].metadata[1].values.back();
    }

    text_t formatkey = toptype + toptitle;
    if (!arg_d.empty()) formatkey = toptype + "Document";

    text_tmap::const_iterator it = collectinfo.format.find(formatkey);
    if (it != collectinfo.format.end()) formatstring = (*it).second;

    // output the table of contents
    if (!arg_d.empty()) {
      
      if (toptype == "Hierarchy")
	output_standard_toc (thistype, formatinfo, formatstring, args, 
			     collectproto, disp, outconvert, textout, logout);

      else
	output_document_toc (args, formatinfo, collectproto, disp,
			     outconvert, textout, logout);

    } else {
      output_standard_toc (thistype, formatinfo, formatstring, args,
			   collectproto, disp, outconvert, textout, logout);
    }

    // output the document text
    textout << "<p>\n";
    output_document (OID, args, collectproto, disp, outconvert, textout, logout);

    textout << outconvert << disp << "_document:footer_\n";
  }
  return true;
}

void documentaction::output_text (const ResultDocInfo_t &docinfo, format_t *formatlistptr,
				  const TermInfo_tarray &terminfo, const text_t &OID,
				  bool highlight, const text_t &hastxt, int wanttext, 
				  text_t &collection, recptproto *collectproto, displayclass &disp, 
				  outconvertclass &outconvert, ostream &textout, 
				  ostream &logout) {
  
  DocumentRequest_t docrequest;
  DocumentResponse_t docresponse;
  comerror_t err;

  if (hastxt == "1") {

    if (wanttext) {
      // get the text
      docrequest.OID = OID;
      collectproto->get_document (collection, docrequest, docresponse, err, logout);

      // cut down on overhead by not using formattools if we only want the text
      // (wanttext will equal 2 if we want text and other stuff too)
      if (wanttext == 1)
	if (highlight)
	  highlighttext(docresponse.doc, terminfo, disp, outconvert, textout, logout);
	else
	  textout << outconvert << disp << docresponse.doc;
    }
  
    if (wanttext != 1) {
      text_t doctext = get_formatted_string (docinfo, formatlistptr, docresponse.doc); 
      
      if (highlight)
	highlighttext(doctext, terminfo, disp, outconvert, textout, logout);
      else
	textout << outconvert << disp << doctext;
    }
  }
}


void documentaction::output_document (const text_t &OID, cgiargsclass &args, 
				      recptproto *collectproto, displayclass &disp, 
				      outconvertclass &outconvert, ostream &textout, 
				      ostream &logout) {
  FilterResponse_t inforesponse;
  FilterResponse_t queryresponse;
  text_tarray metadata;
  bool getParents = false;
  bool highlight = false;
  int wanttext = 0;
  int arg_gt = args.getintarg("gt");
  text_t &collection = args["c"];
    
  // if we have a query string and highlighting is turned on we need
  // to redo the query to get the terms for highlighting
  if (!args["q"].empty() && args.getintarg("hl")) {
    FilterRequest_t request;
    request.filterResultOptions = FRmatchTerms;
    if (do_query (request, args, collectproto, queryresponse, logout))
      highlight = true;
  }

  format_t *formatlistptr = new format_t();
  parse_formatstring (formatinfo.DocumentText, formatlistptr, metadata, getParents);

  metadata.push_back("hastxt");
  metadata.push_back("haschildren");
  
  if (formatinfo.DocumentText == "[Text]")
    wanttext = 1;
  else {
    char *docformat = formatinfo.DocumentText.getcstr();
    if (strstr (docformat, "[Text]") != NULL)
      wanttext = 2;
    delete docformat;
  }
    
  int metasize = metadata.size();

  if (get_info (OID, collection, metadata, getParents, collectproto, inforesponse, logout)) {
    text_t hastxt = inforesponse.docInfo[0].metadata[metasize-2].values.back();
    text_t &haschildren = inforesponse.docInfo[0].metadata[metasize-1].values.back();

    if (arg_gt == 0) {
      output_text (inforesponse.docInfo[0], formatlistptr, queryresponse.termInfo, 
		   OID, highlight, hastxt, wanttext, collection, collectproto, 
		   disp, outconvert, textout, logout);


    } else {

      ResultDocInfo_t thisdocinfo = inforesponse.docInfo[0];
	
      // text is to be expanded
      text_t exOID = OID;
      if (haschildren != "1") exOID = get_parent (OID);
      if (exOID.empty()) exOID = OID;
	
      // if we're not in a document (i.e. we're in a top level classification)
      // we need to pass the "classify" string to get_contents so that it 
      // doesn't recurse all the way through each document in the classification
      text_t classifytype;
      if (args["d"].empty()) classifytype = "classify";

      int i = 0; // not used
      get_contents (exOID, classifytype, metadata, i, collection, 
		    collectproto, inforesponse, logout);
	
      ResultDocInfo_tarray::const_iterator sechere = inforesponse.docInfo.begin();
      ResultDocInfo_tarray::const_iterator secend = inforesponse.docInfo.end();
	
      if (arg_gt == 1) {
	// check if there are more than 10 sections containing text to be expanded - 
	// if there are output warning message - this isn't a great way to do this
	// since the sections may be very large or very small - one day I'll fix it 
	// -- Stefan.
	int seccount = 0;
	while (sechere != secend) {
	  const text_t &shastxt = (*sechere).metadata[metasize-2].values.back();
	  if (shastxt == "1") seccount ++;
	  if (seccount > 10) break;
	  sechere ++;
	}
	if (seccount > 10) {
	  // more than 10 sections so output warning message and text 
	  // for current section only
	  textout << outconvert << disp << "_document:textltwarning_";

	  output_text (thisdocinfo, formatlistptr, queryresponse.termInfo, 
		       OID, highlight, hastxt, wanttext, collection, 
		       collectproto, disp, outconvert, textout, logout);
	  
	}
	else arg_gt = 2;
      }

      if (arg_gt == 2) {
	// get the text for each section
	sechere = inforesponse.docInfo.begin();
	int count = 0;
	while (sechere != secend) {
	  textout << outconvert << disp << "\n<p><a name=" << count << "></a>\n";

	  const text_t &shastxt = (*sechere).metadata[metasize-2].values.back();

	  output_text (*sechere, formatlistptr, queryresponse.termInfo, 
		       (*sechere).OID, highlight, shastxt, wanttext, collection, 
		       collectproto, disp, outconvert, textout, logout);
	  count ++;
	  sechere ++;
	}
      }
    }
  }
  delete formatlistptr;
}

// highlighttext highlights query terms in text string and outputs the resulting text string
void documentaction::highlighttext(text_t &text, const TermInfo_tarray &terms, 
				   displayclass &disp, outconvertclass &outconvert, 
				   ostream &textout, ostream &/*logout*/) {

  text_tmap allterms;
  text_tmap::const_iterator it;

  // first load all the term variations into a map
  TermInfo_tarray::const_iterator this_term = terms.begin();
  TermInfo_tarray::const_iterator last_term = terms.end();
  while (this_term != last_term) {
    text_tarray::const_iterator this_var = (*this_term).matchTerms.begin();
    text_tarray::const_iterator last_var = (*this_term).matchTerms.end();
    while (this_var != last_var) {
      allterms[*this_var] = 1;
      this_var ++;
    }
    this_term ++;
  }

  // get the text to start and end a hightlight
  text_t starthighlight = "<b><u>";
  text_t endhighlight = "</u></b>";
  if (disp.isdefaultmacro("Global", "starthighlight"))
    disp.expandstring("Global", "_starthighlight_", starthighlight);
  if (disp.isdefaultmacro("Global", "endhighlight"))
    disp.expandstring("Global", "_endhighlight_", endhighlight);


  text_t::iterator here = text.begin(); 
  text_t::iterator end = text.end();
  text_t word, buffer;
  while (here != end) {
    if (is_unicode_letdig(*here)) {
      // not word boundary
      word.push_back(*here);
      here++;

    } else {
      // found word boundary
      // add last word if there was one
      if (!word.empty()) {
	it = allterms.find(word);
	if (it != allterms.end()) {
	  word = starthighlight + word + endhighlight;
	}
	buffer += word;
        word.clear();
      }

      if (*here == '<') {
        // skip over rest of html tag
	while ((here != end) && (*here != '>')) {
	  buffer.push_back(*here);
	  here++;
	}
      }

      buffer.push_back(*here);
      here++;

      if (buffer.size() > 1024) {
	textout << outconvert << disp << buffer;
	buffer.clear();
      }
    }
  }
  textout << outconvert << disp << buffer;
}
